# 计算机登录用户: jk
# 系统日期: 2023/11/7 17:41
# 项目名称: chipeak_cv_data_tool
# 开发者: zhanyong
import os
from pathlib import Path
import ast
import json
from ccdt.dataset import *
from collections import defaultdict
import time
import itertools
import hashlib
import ccdt.dataset.utils.labelme_load

def _isArrayLike(obj):
    return hasattr(obj, '__iter__') and hasattr(obj, '__len__')


class CocoToLabelme(BaseLabelme):
    # 当前设计的coco转labelme，主要是图像文件没有对应的注释，注释都在coco文件中
    def __init__(self, *args, **kwargs):
        # 定义labelme保存数据的对象，通过coco从新封装每一张图像，对应的json属性，并追加列表，使用labelme的基类中self.save_labelme()保存方法
        self.input_dir = args[0][0].get('input_dir')
        self.coco_path = args[0][0].get('input_coco_file')  # 获取coco文件路径
        # load dataset
        self.dataset, self.anns, self.cats, self.imgs = dict(), dict(), dict(), dict()
        self.img_to_anns, self.cat_to_imgs = defaultdict(list), defaultdict(list)
        if self.coco_path is not None:
            print('loading annotations into memory...')
            tic = time.time()
            with open(self.coco_path, 'r') as f:
                dataset = json.load(f)
            assert type(dataset) == dict, 'annotation file format {} not supported'.format(type(dataset))
            print('Done (t={:0.2f}s)'.format(time.time() - tic))
            self.dataset = dataset
            self.create_index()  # 创建索引
        super(CocoToLabelme, self).__init__(*args, **kwargs)

    def coco2labelme(self):
        labelme_datasets = list()
        # 获取每一张图片的id
        img_ids = self.get_img_ids()
        for img_id in img_ids:
            dataset = {  # 每次迭代一张图像，就重新初始化
                "image_dir": "00.images",  # 封装图像文件存储目录的相对路径，方便后期路径重组及拼接
                "image_file": "",  # 封装图像文件名称
                "image_width": "",  # 封装图像宽度
                "image_height": "",  # 封装图像高度
                "labelme_dir": "01.labelme",  # 封装json文件存储目录的相对路径，方便重组及拼接
                "labelme_file": "",  # 封装json文件名称
                "input_dir": "",  # 封装处理数据，输入路径目录
                "output_dir": "",  # 封装输出路径目录
                "http_url": "",  # 封装http对象存储服务访问服务地址
                "error_path": "",  # 错误数据存放总目录，不分错误类别，一般为输出路径+error_path拼接，写死的自定义目录。coco转labelme时用不上。
                "labelme_info": None,
                "background": True,  # 封装一张图像属于负样本还是正样本，默认为True，正样本，有标注
                "full_path": "",  # 封装一张图像绝对路径
                "json_path": "",  # 封装一张图像对应json文件绝对路径，用于输出时写文件的路径使用
                "md5_value": "",  # 封装一张图像MD5值，用于唯一性判断
                "relative_path": "",  # 封装图像使用标注工具读取相对路径，格式为：..\\00.images\\000000000419.jpg
                "only_annotation": False,  # 封装是图像还是处理图像对应标注内容的判断条件，默认图片和注释文件一起处理
            }
            image_attr = self.load_imgs(img_id)[0]  # 根据图像索引，获取图像属性
            ann_ids = self.get_ann_ids(img_ids=img_id)  # 根据图像索引，获取注释索引
            anns = self.load_anns(ann_ids)  # 根据注释索引，获取注释属性
            if ann_ids:  # 当注释id不为空的情况下，才进行标注属性封装，才写json文件，否则为背景，无需写json
                obj_path = Path(image_attr.get("file_name"))
                relative_path = os.path.join('..', '00.images', image_attr.get("file_name"))
                json_name = obj_path.stem + ".json"
                image_width = image_attr.get('width')
                image_height = image_attr.get('height')
                dataset.update({'input_dir': self.input_dir})
                dataset.update({'imagePath': relative_path})
                dataset.update({'image_width': image_width})
                dataset.update({'image_height': image_height})
                dataset.update({'labelme_file': json_name})
                dataset.update({'image_file': image_attr.get("file_name")})
                full_path = os.path.join(dataset.get('input_dir'), dataset.get('image_dir'), dataset.get('image_file'))
                json_path = os.path.join(dataset.get('input_dir'), dataset.get('labelme_dir'), dataset.get('labelme_file'))
                md5value = self.calculate_file_md5(full_path, dataset)
                dataset.update({'full_path': full_path})
                dataset.update({'json_path': json_path})
                dataset.update({'md5_value': md5value})
                labelme_data = dict(
                    version='4.5.9',
                    flags={},
                    shapes=[],
                    imagePath=None,
                    imageData=None,
                    imageHeight=None,
                    imageWidth=None,
                    md5Value=None
                )
                shapes = []
                for index, ann in enumerate(anns):  # enumerate为一个可迭代对象，它的作用是为可迭代对象的每个元素分配一个唯一的索引，以便在迭代时轻松获取元素的索引和值。
                    # 取到同一张图片的多个注释属性
                    category_id = ann['category_id']
                    # 获取坐标框
                    bbox = ann['bbox']
                    # 坐标切割
                    bbox = [bbox[0:2], bbox[2:4]]
                    # 左上角的坐标(x,y)右上角的坐标(x,y+h)左下角的坐标(x+w,y)右下角的坐标(x+w,y+h)，一般coco文件中的注释坐标都为中心点坐标。
                    points = [bbox[0], [bbox[0][0] + bbox[1][0], bbox[0][1] + bbox[1][1]]]
                    # 通过类别id获取类别名称
                    cats = self.load_cats(category_id)[0]
                    # 取到类别名称
                    name = cats['name']
                    # 只处理带矩形框，并转labelme
                    shape = {"label": name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}, 'text': None}
                    shapes.append(shape)
                # labelme_data['shapes'] = shapes
                labelme_data.update({'shapes': shapes})
                labelme_data.update({'imagePath': relative_path})
                labelme_data.update({'imageWidth': image_width})
                labelme_data.update({'imageHeight': image_height})
                labelme_data.update({'md5Value': md5value})
                dataset.update({'labelme_info': labelme_data})
                labelme_datasets.append(dataset)
        # 保存labelme数据集合
        self.save_labelme(labelme_datasets, self.output_dir, None)  # self.output_dir为空字符串也是可以的

    def create_index(self):
        # create index
        print('creating index...')
        anns, cats, imgs = {}, {}, {}
        img_to_anns, cat_to_imgs = defaultdict(list), defaultdict(list)
        if 'annotations' in self.dataset:
            for ann in self.dataset['annotations']:
                img_to_anns[ann['image_id']].append(ann)
                anns[ann['id']] = ann

        if 'images' in self.dataset:
            for img in self.dataset['images']:
                imgs[img['id']] = img

        if 'categories' in self.dataset:
            for cat in self.dataset['categories']:
                cats[cat['id']] = cat

        if 'annotations' in self.dataset and 'categories' in self.dataset:
            for ann in self.dataset['annotations']:
                cat_to_imgs[ann['category_id']].append(ann['image_id'])

        print('index created!')

        # create class members
        self.anns = anns
        self.img_to_anns = img_to_anns  # 图像与类别对应关系
        self.cat_to_imgs = cat_to_imgs  # 类别与图像对应关系
        self.imgs = imgs  # 图像属性
        self.cats = cats  # 类别

    def get_img_ids(self, img_ids=None, cat_ids=None):
        """
         Get img ids that satisfy given filter conditions.
        :param img_ids (int array) : get imgs for given ids
        :param cat_ids (int array) : get imgs with all given cats
        :return: ids (int array)  : integer array of img ids
        @param img_ids: 
        @param cat_ids: 
        @return: 
        """
        if cat_ids is None:
            cat_ids = []
        if img_ids is None:
            img_ids = []
        img_ids = img_ids if _isArrayLike(img_ids) else [img_ids]
        cat_ids = cat_ids if _isArrayLike(cat_ids) else [cat_ids]

        if len(img_ids) == len(cat_ids) == 0:
            ids = self.imgs.keys()
        else:
            ids = set(img_ids)
            for i, catId in enumerate(cat_ids):
                if i == 0 and len(ids) == 0:
                    ids = set(self.cat_to_imgs[catId])
                else:
                    ids &= set(self.cat_to_imgs[catId])
        return list(ids)

    def load_imgs(self, ids=None):
        """
        Load anns with the specified ids.
        :param ids (int array)       : integer ids specifying img
        :return: imgs (object array) : loaded img objects
        @param ids: 
        """
        if ids is None:
            ids = []
        if _isArrayLike(ids):
            return [self.imgs[id] for id in ids]
        elif type(ids) == int:
            return [self.imgs[ids]]

    def get_ann_ids(self, img_ids=None, cat_ids=None, area_rng=None, iscrowd=None):
        """
        Get ann ids that satisfy given filter conditions. default skips that filter
        :param img_ids  (int array)     : get anns for given imgs
               cat_ids  (int array)     : get anns for given cats
               area_rng (float array)   : get anns for given area range (e.g. [0 inf])
               iscrowd (boolean)       : get anns for given crowd label (False or True)
        :return: ids (int array)       : integer array of ann ids
        @param iscrowd:
        @param area_rng:
        @param cat_ids:
        @param img_ids:
        """
        if area_rng is None:
            area_rng = []
        if cat_ids is None:
            cat_ids = []
        if img_ids is None:
            img_ids = []
        img_ids = img_ids if _isArrayLike(img_ids) else [img_ids]
        cat_ids = cat_ids if _isArrayLike(cat_ids) else [cat_ids]

        if len(img_ids) == len(cat_ids) == len(area_rng) == 0:
            anns = self.dataset['annotations']
        else:
            if not len(img_ids) == 0:
                lists = [self.img_to_anns[imgId] for imgId in img_ids if imgId in self.img_to_anns]
                anns = list(itertools.chain.from_iterable(lists))
            else:
                anns = self.dataset['annotations']
            anns = anns if len(cat_ids) == 0 else [ann for ann in anns if ann['category_id'] in cat_ids]
            anns = anns if len(area_rng) == 0 else [ann for ann in anns if area_rng[0] < ann['area'] < area_rng[1]]
        if iscrowd is not None:
            ids = [ann['id'] for ann in anns if ann['iscrowd'] == iscrowd]
        else:
            ids = [ann['id'] for ann in anns]
        return ids  # 返回注释索引

    def load_anns(self, ids=None):
        """
        Load anns with the specified ids.
        :param ids (int array)       : integer ids specifying anns        :return:  (object array) : loaded ann objects
        @param ids:
        """
        if ids is None:
            ids = []
        if _isArrayLike(ids):
            return [self.anns[id] for id in ids]
        elif type(ids) == int:
            return [self.anns[ids]]

    def load_cats(self, ids=None):
        """
        Load cats with the specified ids.
        :param ids (int array)       : integer ids specifying cats        :return:  (object array) : loaded cat objects
        @param ids:
        """
        if ids is None:
            ids = []
        if _isArrayLike(ids):
            return [self.cats[id] for id in ids]
        elif type(ids) == int:
            return [self.cats[ids]]

    @staticmethod
    def calculate_file_md5(file_path, dataset):
        """
        functools.lru_cache装饰器对文件的MD5值进行了缓存，暂时没有用
        采用最近最少使用的缓存策略，最多缓存128个不同的文件的MD5值
        这样可以大大减少重复计算MD5值的次数，节约计算资源，提高程序性能。
        """

        # import ccdt.dataset.utils.labelme_load
        # from ccdt.dataset.utils.labelme_load import calculate_sha3_512
        # content = utils.labelme_load.LabelmeLoad.read_file(file_path)
        # sha3_512 = utils.labelme_load.LabelmeLoad.calculate_sha3_512(content)
        # print(sha3_512)
        try:
            with open(file_path, 'rb') as f:
                hasher = hashlib.sha3_512()
                buf = f.read(8192)
                while buf:
                    hasher.update(buf)
                    buf = f.read(8192)
                return hasher.hexdigest()
        except Exception as e:
            print(e)
            print("图片存放目录不存在00.images目录，该目录为标准，必须手动创建后把图像移动到00.images目录中，再次重新执行指令")
            images_path = os.path.join(dataset.get('input_dir'), dataset.get('image_dir'))
            json_path = os.path.join(dataset.get('input_dir'), dataset.get('labelme_dir'))
            os.makedirs(images_path, exist_ok=True)
            os.makedirs(json_path, exist_ok=True)
            print("请在此{}".format(dataset.get('input_dir')) + "   路径下移动图像到00.images目录")
            exit()



# 计算机登录用户: jk
# 系统日期: 2023/10/14 12:36
# 项目名称: chipeak_cv_data_tool
# 开发者: zhanyong
import os
from pathlib import Path
from ccdt.dataset import *


class BaseTxt(BaseLabelme):
    def __init__(self, *args, **kwargs):
        self.txt_path = args[1]  # 获取voc数据集对应的xml目录路径
        self.txt_path_list = LabelmeLoad.get_txt_path(self.txt_path)
        # 在这里定义labelme数据结构格式初始化
        super(BaseTxt, self).__init__(*args, **kwargs)

    def txt2labelme(self):
        txt_to_labelme = list()
        # 每循环一次，拼接对应的xml文件路径，直接读取xml文件，然后转换成labelme_info
        for dataset in self.datasets:
            judge_key = dataset.get("image_dir").split("/")[0]  # 获取唯一判断标识，视频目录不允许嵌套，否则唯一判断关系会错乱
            obj_path = Path(dataset.get('image_file'))  # 获取文件前缀
            lines_list = list()
            for txt_path in self.txt_path_list:
                txt_path.items()
                for key, value in txt_path.items():
                    if key == judge_key:
                        # 使用 readlines 方法读取文件内容，并去除每行末尾的换行符
                        with open(value, 'r') as file:
                            lines_list = [line.rstrip() for line in file.readlines()]
            find_shape_list = list()
            for line in lines_list:
                # 使用 zfill 方法将字符串往左补零到指定长度
                result = line.split(",")[0].zfill(8)
                if obj_path.stem == result:
                    find_shape_list.append(line)
                    break  # 找到唯一存在的那一帧图像后，就结束循环
            # 检查列表是否为空，为空就是没有找到当背景处理，不写json文件
            if len(find_shape_list) != 0:  # 不为空就是在tex文本中找到了标注，写入当前帧图像对应的标注json文件
                labelme_info = self.analysis_txt(find_shape_list, dataset.get("image_width"), dataset.get("image_height"), dataset.get("relative_path"))
                dataset.update({'background': True})
                dataset.update({'labelme_info': labelme_info})
            txt_to_labelme.append(dataset)
        self.save_labelme(txt_to_labelme, self.output_dir, None)  # self.output_dir为空字符串也是可以的

    @staticmethod
    def analysis_txt(find_shape, w, h, file_name):
        # image_path = os.path.join("../", "00.images", file_name).replace('\\', '/')
        # 定义labelme数据结构
        labelme_data = dict(
            version='4.5.14',
            flags={},
            shapes=[],
            imagePath=None,
            imageData=None,
            imageHeight=None,
            imageWidth=None
        )
        for shape in find_shape:
            xmin = int(shape.split(",")[2])
            ymin = int(shape.split(",")[3])
            xmax = int(shape.split(",")[4])
            ymax = int(shape.split(",")[5])
            name = shape.split(",")[6]
            points = [[xmin, ymin], [xmax, ymax]]  # 坐标点计算，目前使用左上角的点和右下角的点计算
            make_up_shape = {"label": name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}, 'text': None}
            labelme_data.get('shapes').append(make_up_shape)  # 这里是确定了每一张图像只标注一个，如果是标注多个的话，就是一个列表了
        # labelme_data.update({"imagePath": image_path})
        labelme_data["imagePath"] = file_name
        labelme_data["imageWidth"] = w
        labelme_data["imageHeight"] = h
        return labelme_data



# 计算机登录用户: jk
# 系统日期: 2023/5/18 15:58
# 项目名称: async_ccdt
# 开发者: zhanyong
import shutil
import asyncio
import os
from concurrent.futures import ThreadPoolExecutor
from tqdm import tqdm
from pathlib import Path
import argparse
import json
import aiofiles
from ccdt.dataset.utils.encoder import Encoder


class AsyncIoTask(object):
    """
    asyncio.get_running_loop().run_in_executor函数的作用是将指定函数在线程池中异步执行，返回异步任务future。这样我们可以在多个任务之间切换，并且不需要等待文件写入、文件复制和文件移动操作完成，从而实现异步高并发和非阻塞操作。
    run_in_executor(None)，None参数表示开启默认线程执行，异步执行 CPU 密集型任务或 I/O 密集型任务
    """

    # def __init__(self, max_concurrent=1024):  # 设置最大并发数量
    #     self.semaphore = asyncio.Semaphore(max_concurrent)
    # @staticmethod
    # async def write_file(file_path: str, content: dict):
    #     # 通过 aiofiles.open() 实现异步方式进行文件的读写操作
    #     async with aiofiles.open(file_path, 'w') as f:
    #         # 使用 aiofiles 写入文件
    #         await f.write(json.dumps(content, indent=2, cls=Encoder))
    #
    # @staticmethod
    # async def copy_file(src_file_path: str, dst_file_path: str):
    #     try:
    #         async with aiofiles.open(src_file_path, 'rb') as src_file, \
    #                 aiofiles.open(dst_file_path, 'wb') as dst_file:
    #             # 使用 aiofiles 复制文件内容
    #             async for chunk in src_file.iter_chunked(1024 * 1024):  # Adjust chunk size as needed
    #                 await dst_file.write(chunk)
    #     except PermissionError as e:
    #         # 处理权限被拒绝的情况
    #         print(f"PermissionError: {e}")
    #         # 执行其他错误处理或者记录日志的操作
    #
    # @staticmethod
    # async def move_file(src_file_path: str, dst_file_path: str):
    #     # 异步移动文件
    #     async with aiofiles.open(src_file_path, 'rb') as src_file, \
    #             aiofiles.open(dst_file_path, 'wb') as dst_file:
    #         # 使用 aiofiles 移动文件内容
    #         async for chunk in src_file.iter_chunked(1024 * 1024):  # Adjust chunk size as needed
    #             await dst_file.write(chunk)
    #     # 删除源文件，如果需要
    #     os.remove(src_file_path)

    @staticmethod
    async def write_file(file_path: str, content: dict):
        # try:
        # 通过aiofiles.open(),实现异步方式进行文件的读写操作
        async with aiofiles.open(file_path, 'w') as f:
            # cls=json.JSONEncoder不是必须的，因为json.dumps() 函数会自动选择适当的编码器来对 Python 对象进行编码
            await f.write(json.dumps(content, indent=2, cls=Encoder))
        # finally:
        #     await f.close()  # 资源释放和回收

    @staticmethod
    async def copy_file(src_file_path: str, dst_file_path: Path):
        # 异步复制文件
        await asyncio.get_running_loop().run_in_executor(None, shutil.copy, src_file_path, dst_file_path)

    @staticmethod
    async def move_file(src_file_path: str, dst_file_path: Path):
        # 异步移动文件
        await asyncio.get_running_loop().run_in_executor(None, shutil.move, src_file_path, dst_file_path)

    async def process_files(self, path_list, judge_dir, index, custom_label):
        """
        异步并发处理列表中的文件路径
        :param path_list: 文件路径列表
        :param judge_dir: 判断真假值，如果为真就在指定输出目录下，拷贝重写文件，如果为假或空值，就在输入路径下重写文件
        :param index: 索引值传递，用于自定义目录
        :param custom_label: 标签名称
        """
        # 线程池大小以当前计算机CPU逻辑核心数为准
        thread_pool_size = os.cpu_count() or 1
        # 线程池中的协程任务异步高并发
        with ThreadPoolExecutor(thread_pool_size) as executor:
            # 一次性提交多个协程任务
            tasks = []
            print(f'迭代异步任务开始')
            for data_info in tqdm(path_list):
                if judge_dir:  # 传递输出路径的表示要拷贝数据，不传递表示就在输入路径下重写
                    obj_path = Path(data_info.get('image_file'))  # 初始化文件为对象
                    # 如果图片名称后缀格式重复多次，就进行重写json文件后保存，重命名图片名称
                    if data_info['image_file'].count(obj_path.suffix) >= 2:
                        file_path = data_info.get('full_path')
                        print(f'图像数据文件格式存在双后缀：{file_path}  请核对数据')
                    else:  # 处理正常数据，包含重构列表原始封装列表
                        if isinstance(judge_dir, str) or isinstance(index, bool) or isinstance(index, str) or isinstance(index, argparse.Namespace):
                            if custom_label:
                                save_images_dir = str(Path(data_info['output_dir'], custom_label, data_info['image_dir']))
                                os.makedirs(save_images_dir, exist_ok=True)
                                save_labelme_dir = str(Path(data_info['output_dir'], custom_label, data_info['labelme_dir']))
                                os.makedirs(save_labelme_dir, exist_ok=True)
                            else:
                                save_images_dir = Path(data_info['output_dir'], data_info['image_dir'])
                                os.makedirs(save_images_dir, exist_ok=True)
                                save_labelme_dir = Path(data_info['output_dir'], data_info['labelme_dir'])
                                os.makedirs(save_labelme_dir, exist_ok=True)
                            # 图像文件处理，json文件处理
                            if index is True:  # 文件移动
                                tasks.append(asyncio.create_task(self.move_file(data_info['full_path'], save_images_dir)))
                                if data_info.get('background') is True:  # 不为背景才移动json文件
                                    tasks.append(asyncio.create_task(self.move_file(data_info['original_json_path'], save_labelme_dir)))
                            else:  # 文件拷贝
                                # 使用 os.path.normpath() 函数，将 Windows 路径转换成标准的跨平台的路径格式
                                if os.path.normpath(data_info['input_dir']) == os.path.normpath(data_info['output_dir']):
                                    pass
                                else:
                                    tasks.append(asyncio.create_task(self.copy_file(data_info['full_path'], save_images_dir)))
                                if data_info['background']:  # 表示不为负样本，有标注存在
                                    tasks.append(asyncio.create_task(self.write_file(data_info['json_path'], data_info.get('labelme_info'))))
                        else:
                            if isinstance(index, int) and isinstance(custom_label, str):
                                custom_dir = '{:0>4d}'.format(index) + '_' + custom_label
                                rebuild_dir = os.path.join(data_info['output_dir'], custom_dir)
                                extract_images_dir = Path(rebuild_dir, data_info['image_dir'])
                                os.makedirs(extract_images_dir, exist_ok=True)
                                extract_labelme_dir = os.path.join(rebuild_dir, data_info['labelme_dir'])
                                os.makedirs(extract_labelme_dir, exist_ok=True)
                                tasks.append(asyncio.create_task(self.copy_file(data_info['full_path'], extract_images_dir)))
                                if data_info['background']:  # 表示不为负样本，有标注存在
                                    rebuild_json_path = os.path.join(rebuild_dir, data_info['labelme_dir'], data_info['labelme_file'])
                                    tasks.append(asyncio.create_task(self.write_file(rebuild_json_path, data_info.get('labelme_info'))))
                            else:
                                print(f'自定义目录格式不符合要求{custom_label}，请重新输入字符串')
                                exit()
                else:  # 直接重写输入路径下的json文件，不用拷贝一份
                    if data_info['background']:  # 表示不为负样本，有标注存在
                        if index is True:  # 文件移动
                            tasks.append(asyncio.create_task(self.move_file(data_info['json_path'], data_info.get('output_dir'))))
                        else:  # 文件重写
                            # json.dumps() 将 Python 对象转化为 JSON 格式的字符串
                            tasks.append(asyncio.create_task(self.write_file(data_info['json_path'], data_info.get('labelme_info'))))
            print(f'并发处理任务开始')
            await asyncio.gather(*tasks)  # 并发处理文件，会等待所有的异步任务完成

    # async def process_file_with_semaphore(self, judge_dir, data_info, index, custom_label):
    #     semaphore = 1024
    #     async with semaphore:
    #         tasks = []
    #         if judge_dir:  # 传递输出路径的表示要拷贝数据，不传递表示就在输入路径下重写
    #             obj_path = Path(data_info.get('image_file'))  # 初始化文件为对象
    #             # 如果图片名称后缀格式重复多次，就进行重写json文件后保存，重命名图片名称
    #             if data_info['image_file'].count(obj_path.suffix) >= 2:
    #                 file_path = data_info.get('full_path')
    #                 print(f'图像数据文件格式存在双后缀：{file_path}  请核对数据')
    #             else:  # 处理正常数据，包含重构列表原始封装列表
    #                 if isinstance(judge_dir, str) or isinstance(index, bool) \
    #                         or isinstance(index, str) or isinstance(index, argparse.Namespace):
    #                     save_images_dir = os.path.join(data_info['output_dir'], data_info['image_dir'])
    #                     os.makedirs(save_images_dir, exist_ok=True)
    #                     save_labelme_dir = os.path.join(data_info['output_dir'], data_info['labelme_dir'])
    #                     os.makedirs(save_labelme_dir, exist_ok=True)
    #                     # 图像文件处理，json文件处理
    #                     if index is True:  # 文件移动
    #                         tasks.append(asyncio.create_task(
    #                             self.move_file(data_info['full_path'], save_images_dir)))
    #                         if data_info.get('background') is True:  # 不为背景才移动json文件
    #                             tasks.append(asyncio.create_task(self.move_file(data_info['original_json_path'], save_labelme_dir)))
    #                     else:  # 文件拷贝
    #                         # 使用 os.path.normpath() 函数，将 Windows 路径转换成标准的跨平台的路径格式
    #                         if os.path.normpath(data_info['input_dir']) == os.path.normpath(data_info['output_dir']):
    #                             pass
    #                         else:
    #                             tasks.append(asyncio.create_task(self.copy_file(data_info['full_path'], save_images_dir)))
    #                         if data_info['background']:  # 表示不为负样本，有标注存在
    #                             tasks.append(asyncio.create_task(self.write_file(data_info['json_path'], data_info.get('labelme_info'))))
    #                 else:
    #                     if isinstance(index, int) and isinstance(custom_label, str):
    #                         custom_dir = '{:0>4d}'.format(index) + '_' + custom_label
    #                         rebuild_dir = os.path.join(data_info['output_dir'], custom_dir)
    #                         extract_images_dir = os.path.join(rebuild_dir, data_info['image_dir'])
    #                         os.makedirs(extract_images_dir, exist_ok=True)
    #                         extract_labelme_dir = os.path.join(rebuild_dir, data_info['labelme_dir'])
    #                         os.makedirs(extract_labelme_dir, exist_ok=True)
    #                         tasks.append(asyncio.create_task(self.copy_file(data_info['full_path'], extract_images_dir)))
    #                         if data_info['background']:  # 表示不为负样本，有标注存在
    #                             rebuild_json_path = os.path.join(rebuild_dir, data_info['labelme_dir'], data_info['labelme_file'])
    #                             tasks.append(asyncio.create_task(self.write_file(rebuild_json_path, data_info.get('labelme_info'))))
    #                     else:
    #                         print(f'自定义目录格式不符合要求{custom_label}，请重新输入字符串')
    #                         exit()
    #         else:  # 直接重写输入路径下的json文件，不用拷贝一份
    #             if data_info['background']:  # 表示不为负样本，有标注存在
    #                 if index is True:  # 文件移动
    #                     tasks.append(asyncio.create_task(self.move_file(data_info['json_path'], data_info.get('output_dir'))))
    #                 else:  # 文件重写
    #                     # json.dumps() 将 Python 对象转化为 JSON 格式的字符串
    #                     tasks.append(asyncio.create_task(self.write_file(data_info['json_path'], data_info.get('labelme_info'))))
    #         await asyncio.gather(*tasks)  # 会等待所有的异步任务完成
    #
    # async def process_files(self, path_list, judge_dir, index, custom_label):
    #     """
    #     @param path_list: 文件路径列表或者路径传参
    #     @param judge_dir: 判断真假值，如果为真就在指定输出目录下，拷贝重写文件，如果为假或空值，就在输入路径下重写文件
    #     @param index: 索引值传递，用于自定义目录所用
    #     @param custom_label: 标签名称参数传递
    #     """
    #     thread_pool_size = os.cpu_count() or 1
    #     with ThreadPoolExecutor(thread_pool_size) as executor:
    #         tasks = []
    #         print(f'迭代异步任务开始')
    #         for data_info in tqdm(path_list):
    #             if judge_dir:
    #                 tasks.append(asyncio.create_task(self.process_file_with_semaphore(judge_dir, data_info, index, custom_label)))
    #             else:
    #                 if data_info['background']:
    #                     tasks.append(asyncio.create_task(self.process_file_with_semaphore(judge_dir, data_info, index, custom_label)))
    #         print(f'并发处理任务开始')
    #         await asyncio.gather(*tasks)



# 计算机登录用户: jk
# 系统日期: 2023/5/17 9:45
# 项目名称: async_ccdt
# 开发者: zhanyong
import argparse
import ast
import asyncio
from ccdt.dataset import *
import time


def parser_args():
    parser = argparse.ArgumentParser()
    # input_datasets 是必须要传递的参数，可以是包含多个数据集路径的列表字典格式。
    parser.add_argument('input_datasets', type=str, help="labelme数据集路径、coco数据集路径，列表字典传参")
    parser.add_argument('--output-dir', type=str, help="保存路径")
    parser.add_argument('--output-format', type=str, help="输出功能格式，有labelme、coco")
    parser.add_argument('-f', '--function', type=str, required=True, help="功能参数:print,convert,filter,matting,rename,visualize,merge，只能输入单个")
    parser.add_argument('--filter-label', type=ast.literal_eval, help="类别筛选参数，单个与多个都可以输入")
    # 当不输入--only_annotation，默认为False；输入--only_annotation，才会触发True值。False处理labelme和图片，True只处理labelme
    parser.add_argument('--only-annotation', action="store_true", help="默认False，是否只处理注释文件。是为True，否为False")
    parser.add_argument('--filter-shape-type', type=ast.literal_eval, help="形状筛选参数，单个与多个都可以输入")
    parser.add_argument('--input-coco-file', type=str, help="输入形状筛选参数，单个与多个都可以输入")
    parser.add_argument('--rename-attribute', type=ast.literal_eval, help="属性重命名，包含label、flags")
    parser.add_argument('--select-empty', action="store_true", help="默认False，是否保留背景类。是为True，否为False")
    parser.add_argument('--only-select-empty', action="store_true", help="默认False，是否只筛选背景数据。是为True，否为False")
    parser.add_argument('--only-select-shapes', action="store_true", help="默认False，是否只筛选标注有框的数据。是为True，否为False")
    parser.add_argument('--shapes-attribute', type=str, help="筛选属性，包含label（类别）、shape_type（类别形状）、flags（类别属性）")
    parser.add_argument('--filter-flags', type=ast.literal_eval, help="类别属性筛选，输入类别属性字典列表。比如person类下有，手、脚、头")
    parser.add_argument('--file-formats', default=['.jpg', '.jpeg', '.png', '.JPEG', '.JPG', '.PNG', '.webp'], type=str,
                        help="文件格式,其中,'.bmp','.gif','.tif',格式图像不做处理")
    parser.add_argument('--filter-combin', action="store_true", help="是否组合筛选，是为True，否为False")
    parser.add_argument('--extract-portion', type=int, help='按照指定份数平均抽取，比如400张图像，抽取10分，每份40张')
    parser.add_argument('--extract-text', type=ast.literal_eval, help='按照text字段的文本内容抽取')
    parser.add_argument('--select-cut', action="store_true", help="默认False即拷贝，是拷贝还是剪切。是为True，否为False")
    parser.add_argument('--judging-group-id-num', action="store_true", help="默认False即拷贝，有judging-group-id-num参则改变为True，否为False")
    parser.add_argument('--extract-amount', type=int, help='按照指定数量抽取，比如400张图像，抽取100张')
    parser.add_argument('--print-more', action="store_true", help="打印详细信息")
    parser.add_argument('--del-label', type=ast.literal_eval, help="删除label标签")
    parser.add_argument('--judging-letter', type=ast.literal_eval, help="判断车牌字符串中是否存在，I，o,Q")
    parser.add_argument('--label-name', type=str, help="自定义label标签，用于抽取份数时区别标注目录")
    parser.add_argument('--http-url', type=str, help="minio文件对象存储中，网络文件统一资源定位器，http://192.168.1.235:9393/chipeak-dataset")
    parser.add_argument('--min-pixel', type=int, default=512, help='最小像素截图设置，默认512像素。即大于512像素的矩形框才进行截图')
    parser.add_argument('--judging-group', type=int, help='默认值为2个shape元素为一组，用于判断分组元素的数量')
    parser.add_argument('--judging-flags', type=ast.literal_eval, help="检查flags默认标注属性，是否符合标注准则")
    parser.add_argument('--judging-label', type=str, help="检查车牌默认分组属性，是否符合标注准则，车牌一定要打组，如果没有打组就筛选出来")
    parser.add_argument('--judging-polygon', type=int, help='检查多边形标注的点是否超出预期数量，比如4个点的多边形，不能出现5个')
    parser.add_argument('--judging-cross-the-border', type=str, help="检查标注形状是否超越原始图像边界")
    parser.add_argument('--point-number', type=int, help='点标注的数量，用于标注点排序时，追加标注点到列表中然后判断，是否满足标注规则')
    parser.add_argument('--automatic-correction', action="store_true", help="默认False，是否自动矫正标注形状超越图像边界情况。是为True，否为False")
    parser.add_argument('--rectangle-merge', action="store_true", help="默认False，填写参数代表为true，用于判断合并条件，该条件表示矩形框合并。是为True，否为False")
    parser.add_argument('--sync', action="store_true", help="默认False，填写参数代表为true，用于判断是同步处理，还是异步处理。是为True，否为False")
    parser.add_argument('--threshold', type=float, help="阈值参数，模型预测数据集，设定的阈值")
    parser.add_argument('--leak-check', action="store_true", help="默认False，填写参数代表为true，用于筛选漏检出数据。是为True，否为False")
    parser.add_argument('--randomize', action="store_true", help="默认False，填写参数代表为true，用于列表随机化。是为True，否为False")
    parser.add_argument('--test-ratio', type=float, default=0.2, help='默认测试集比例值')
    parser.add_argument('--train-ratio', type=float, default=0.8, help='默认训练集比例值')
    parser.add_argument('--error-check', action="store_true", help="默认False，填写参数代表为true，用于筛选误检出数据。是为True，否为False")
    parser.add_argument('--right-check', action="store_true", help="默认False，填写参数代表为true，用于筛选正确检出数据。是为True，否为False")
    parser.add_argument('--coco-to-bbox', action="store_true", help="默认False，填写参数代表为true，用于判断多边形转矩形框。是为True，否为False")
    parser.add_argument('--background', action="store_true", help="默认False，填写参数代表为true，用于把系统输出的json转换为labelme对应的json。是为True，否为False")
    parser.add_argument('--is-PCA', action="store_true",
                        help="默认False，填写参数代表为true，用于直方图降维条件判断。是为True，但会造成信息丢失，否为False，信息更准确")
    parser.add_argument('--delete', action="store_true", help="默认False，填写参数代表为true，用于删除勾选数据。是为True，否为False")
    parser.add_argument('--flags-true', action="store_true", help="默认False，填写参数代表为true，用于删除勾选数据。是为True，否为False")
    parser.add_argument('--opt', type=str, help="opt值为local表示局部直方图计算，opt值为global表示全局直方图计算")
    parser.add_argument('--json-file', type=str, help="图像文件约定目录规则,00.images")
    parser.add_argument('--image-file', type=str, help="json文件约定目录规则，01.labelme")
    parser.add_argument('--helmet-true', type=str, help="分类目录规则,helmet")
    parser.add_argument('--helmet-false', type=str, help="分类目录规则，no_select")
    parser.add_argument('--mv5-value', type=str, help="分类目录规则，no_select")
    parser.add_argument('--flags-value', type=ast.literal_eval, help="填写flags的字典值")
    parser.add_argument('--image-check', action="store_true",
                        help="默认False，填写参数代表为true即开启检查功能并移除图像，用于检查图像格式、通道、完整性及其它问题。是为True，否为False")
    args = parser.parse_args()

    if args.function == 'filter_positive':  # 筛选正样本
        return args
    elif args.function == 'filter_negative':  # 筛选负样本
        return args
    elif args.function == 'filter_label':  # 筛选负样本
        return args
    elif args.function == 'filter_images':  # 筛选训练样本
        return args
    elif args.filter_label and args.function == 'filter_label':  # 按照标注目标的标签进行筛选
        return args
    elif args.filter_flags and args.function == 'filter_flags':  # 按照标注目标的flags属性筛选
        return args
    elif args.filter_shape_type and args.function == 'filter_shape_type':  # 按标注形状进行筛选
        return args
    # 重命名
    elif args.rename_attribute and args.function == 'rename':  # 标注属性重命名，包含label标签、flags、
        return args
    # labelme转coco，coco转labelme
    elif args.function == 'convert':
        return args
    # 抠图，单数据集、多数据集
    elif args.function == 'matting':
        return args
    # 可视化
    elif args.function == 'visualize':
        return args
    # 合并类别筛选数据
    elif args.function == 'merge':
        return args
    elif args.function == 'relation':  # 寻找shape标注形状包含关系，大矩形框包含小多边形
        return args
    elif args.function == 'print':  # 打印labelme标注信息，图像属性信息
        return args
    elif args.function == 'check_image_path':  # 检查标注路径
        return args
    elif args.function == 'delete':  # 按照标注标签删除标注
        return args
    elif args.function == 'extract':  # 抽取labelme数据集，包含按照指定份数抽取，按照图像张数抽取，可以拷贝、剪切
        return args
    elif args.function == 'duplicate':  # 对数据集去重
        return args
    elif args.function == 'compress':  # 对抽取数据集进行压缩
        return args
    elif args.function == 'check':  # 检查分组标注常见错误，包含：一组标注少一个标注框或点，一组标注的group_id值不对。
        return args
    elif args.function == 'correct' or args.function == 'original':  # 对多边形车牌标注进行排序，截取图像，矫正形状摆放位置
        return args
    elif args.function == 'cross':  # 针对标注形状超越图像边界情况，使用程序自动矫正
        return args
    elif args.function == 'pinyin':  # 汉字转拼音
        return args
    elif args.function == 'IOU':  # 标注数据与模型预测数据进行比较，给出漏检、误检、检出、完全预测正确、部分预测正确、完全预测错误、部分预测错误
        return args
    elif args.function == 'compare':  # 误检与漏检比较功能，把MD5值相同的留下
        return args
    elif args.function == 'threshold_filter':  # 阈值筛选，根据不同的阈值挑选不同的labelme数据集
        return args
    elif args.function == 'Bhattacharyya':  # 图像内容相似度去重,计算直方图之间的距离，使用Bhattacharyya（巴氏距离）距离，它衡量了两个概率分布之间的相似程度，它直接基于直方图的概率分布计算。
        return args
    elif args.function == 'mosaic':  # 马赛克
        return args
    elif args.function == 'create_dir':  # 创建00.images和01.labelme目录规则
        return args
    elif args.function == 'flags':  # 根据目录判断，设置flags属性
        return args
    elif args.function == 'md5':  # 对每张图像的标注文件赋值MD5
        return args
    elif args.function == 'video_mark_convert_labelme':  #
        return args
    elif args.function == 'change':  # 把系统输出格式，转换为labelme标注工具可读格式。
        return args
    elif args.function == 'rename_file_name':  # 把中文文件名称，重命名为英文
        return args
    elif args.function == 'soft_deletion':  # 把json数据删除，属于软删除，等同于只拷贝一份图像，原始数据保留
        return args
    elif args.function == 'crop':  # 把目标检测标注的矩形框，截取图像
        return args
    elif args.function == 'check_format':  # 检查图像是否存在，格式不对、通道不对、内容完整性不对等问题
        return args
    elif args.function == 'move':  # 移动文件，临时功能
        return args
    elif args.function == 'modify_flags':  # 修改flags字典值
        return args
    elif args.function == 'filter_flags_true':  # 筛选flags字典值为真的数据
        return args
    elif args.function == 'ratio':  # 筛选flags字典值为真的数据
        return args
    elif args.function == 'pinyin_images':  # 针对分类图像数据，没有labelme的情况，把文件夹中文转换为拼音
        return args
    elif args.function == 'category_name':  # 打印分类类别名称
        return args
    else:
        assert not args.function, '传入的操作功能参数不对:{}'.format(args.function)


def main():
    # async def main():
    args = parser_args()
    # print(args)
    # 把字符串中，转义字符转换成，列表内存储字典元素.
    input_datasets_list = ast.literal_eval(args.input_datasets.replace('\\', '/'))
    if args.sync:  # 同步读写数据处理
        data_info = LabelmeLoad(args, input_datasets_list)  # 初始化输入参数
        if args.function == 'compress':  # 对抽取数据集进行压缩。数据压缩无需对数据进行封装后操作
            async_time = time.time()
            zip_package = data_info.compress_labelme()  # 封装压缩路径及压缩对象
            data_info.make_compress(zip_package)  # 开始压缩
            print('数据读取、压缩使用同步计算耗时')
            print(time.time() - async_time)
        if args.function == 'pinyin':
            data_info.hanzi_to_pinyin()  # 对中文路径转拼音后，重新输出
        if args.function == 'pinyin_images':
            data_info.hanzi_to_pinyin_images()  # 对中文路径转拼音后，只处理有图像的情况，没有labelme
        if args.function == 'check_format':  # 检查图像格式
            data_info.check_images_format()
        if args.function == 'category_name':  # 检查图像格式
            data_info.print_icategory_name(args)

    else:  # 异步读写数据处理，存在少部分同步写数据处理
        async_time = time.time()
        data_info = LabelmeLoad(args, input_datasets_list)  # 初始化输入参数
        # 获取当前正在运行的事件循环，从而可以将异步任务添加到事件循环中执行。在等待IO操作完成的同时，利用CPU计算力进行其他计算，从而提高计算效率
        dataset_info = asyncio.run(data_info.recursive_walk(input_datasets_list[0].get('input_dir'), args.function))
        print('数据读取使用异步计算耗时')
        print(time.time() - async_time)
        dataset = BaseLabelme(dataset_info)  # 初始化labelme基类
        if args.function == 'merge':  # 合并功能
            if args.rectangle_merge:  # 删除人工标注矩形框，然后与模型预测出来的矩形框进行合并功能
                # 加载模型预测的labelme数据集
                model_dataset_info = asyncio.run(data_info.recursive_walk(input_datasets_list[0].get('model_input_dir'), args.function))
                # 先删除人工标注的矩形框
                # original_dataset_info = dataset.del_label(args.del_label)
                dataset.labelme_rectangle_merge(args, model_dataset_info)
            else:
                dataset.merge_labelme(args)  # 人工标注数据，根据label拆分后，自动合并实现功能
            # BaseLabelme.merge(datasets)
        elif args.function == 'matting':  # 抠出标注位置保存labelme
            dataset.intercept_coordinates()
        elif args.function == 'duplicate':  # 对labelme数据集进行去重，重复的图片就删除
            dataset.duplicate_images()
        elif args.function == 'crop':
            dataset.crop_objs(args)  # 目标检测矩形框，截取图像
        elif args.function == 'convert':  # 转换功能，包含labelme转coco，coco转labelme
            if args.output_format == 'labelme':  # coco转labelme
                # 由于dataset_info封装的数据集，无法快速索引，在coco转labelme的时候，从新封装该对象，无需传入labelme基类中
                coco_to_labelme = CocoToLabelme(input_datasets_list)
                coco_to_labelme.coco2labelme()
                # dataset.save_labelme(args.output_dir, None)  # 如果输出路径为空，就直接修改输入目录下的json文件，不为空则重新拷贝图像文件与重写json文件
            elif args.output_format == 'coco':  # labelme转coco
                coco = BaseCoco(dataset_info)
                coco.self2coco(args)
            elif args.output_format == 'voc':  # voc转labelme
                voc = BaseVoc(dataset_info, input_datasets_list[0].get('input_xml_dir'))
                voc.voc2labelme()
            elif args.output_format == 'txt':  # 视频标注转labelme，txt转labelme
                txt = BaseTxt(dataset_info, input_datasets_list[0].get('input_txt_dir'))
                txt.txt2labelme()
            elif args.output_format == 'sys':  # 把系统输出格式，转换为labelme标注工具可读格式。
                sys = BaseSys(dataset_info, args)
                sys.sys2labelme()
        elif args.function == 'rename':  # 重命名label标签功能
            dataset.rename_labelme(args)
            # dataset.save_labelme(args.output_dir, None, None)
        # elif args.function == 'visualize':  # 可视化功能
        # dataset.visualization(args.output_dir)
        elif args.function == 'filter_positive':  # 筛选正样本
            dataset.filter_positive(args)
        elif args.function == 'filter_negative':  # 筛选负样本
            dataset.filter_positive(args)
        elif args.function == 'filter_images':  # 筛选训练样本数据
            dataset.filter_images(args)
        elif args.function == 'filter_label':  # 筛选，指定label标签数据集，默认正样本
            dataset.filter_label(args)
        elif args.function == 'filter_flags':  # 筛选，标注label下的flags类别数据集，默认正样本
            dataset.filter_flags(args)
        elif args.function == 'print':  # 打印功能
            dataset.__repr__()
        elif args.function == 'check_image_path':  # 检查image_path功能
            dataset.check_image_path(args)
        elif args.function == 'delete':  # 删除指定标签类别标注数据集
            dataset.del_label(args.del_label)
            print(f'保存指定label类别进行删除的labelme数据集')
            dataset.save_labelme(args.output_dir, None, None)
        elif args.function == 'extract':  # 抽取labelme数据集功能，指定份数抽取，只允许拷贝；指定图像张数抽取，允许剪切、拷贝
            dataset.extract_labelme(args)
        elif args.function == 'relation':  # 寻找标注形状包含关系，进行自动打组
            dataset.relation_labelme(args)
        elif args.function == 'check':  # 检查分组标注常见错误，包含：一组标注少一个标注框或点，一组标注的group_id值不对。
            dataset.check_group_labelme(args)
        # correct为排序并矫正后的车牌截取，original为不排序也不矫正的车牌截取
        elif args.function == 'correct' or args.function == 'original':  # 排序，按照多边形，左上、右上、右下、左下的顺序排列4个顶点，截取图像，矫正形状摆放位置
            dataset.sort_correct_labelme(args)
        elif args.function == 'cross':  # 针对标注形状超越图像边界情况，使用程序自动矫正
            dataset.cross_boundary_correction(args)
        elif args.function == 'pinyin':  # 针对标注形状超越图像边界情况，使用程序自动矫正
            dataset.hanzi_to_pinyin()
        elif args.function == 'threshold_filter':  # 阈值筛选labelme数据集
            dataset.threshold_filter_labelme(args)
        elif args.function == 'Bhattacharyya':  # 图像内容相似度去重，计算直方图之间的距离，使用Bhattacharyya（巴氏距离）距离，它衡量了两个概率分布之间的相似程度，它直接基于直方图的概率分布计算。
            dataset.bhattacharyya_filter_labelme(args)
        # TP = 标注数据集有标注框，模型预测为有标注框，并且iou重合度高。=正确检出
        # FP = 标注数据集没有标注框，模型预测为有标注框。=误检出
        # FN = 标注数据集有标注框，模型预测为没有标注框。=漏检出
        # TN = 标注数据集没有标注框，模型预测结果也没有标注框。=背景

        # TP：模型正确检测到的正样本数量（即检测正确且标签为1的样本数）
        # FP：模型错误地将负样本检测为正样本的数量（即检测错误且标签为0的样本数）
        # FN：模型没能检测到的正样本数量（即检测错误且标签为1的样本数）
        # TN：模型正确地将负样本检测为负样本的数量（即检测正确且标签为0的样本数）

        # 召回率（Recall，也称为灵敏度或真阳性率）：Recall = TP / (TP + FN)，70 / (70 + 8)
        # 精确率（Precision，也称为查准率）：Precision = TP / (TP + FP)，70 / (70 + 0)
        # 准确率 Accuracy = (TP + TN) / (TP + FN + FP + TN)，（70 + 26） / （70 + 8 + 26 + 0）
        elif args.function == 'IOU':  # 把模型预测结果与人工标注结果进行对比，计算模型预测结果的漏检、误检、检出
            # 加载模型预测的labelme数据集
            model_dataset_info = asyncio.run(data_info.recursive_walk(input_datasets_list[0].get('model_input_dir'), args))
            # 加载标注数据集
            dataset.model_to_iou(model_dataset_info, args)
        elif args.function == 'compare':  # 数据集比较功能，目前比较漏检和正确检出图像相同的情况
            # 加载正确检出数据集
            right_check_dataset_info = asyncio.run(data_info.recursive_walk(input_datasets_list[0].get('model_input_dir'), args))
            # 加载漏检出数据集
            dataset.compare_labelme(right_check_dataset_info, args)
        elif args.function == 'mosaic':
            dataset.chipeak_mosaic()
        elif args.function == 'create_dir':
            dataset.chipeak_make_dir(args)
        elif args.function == 'flags':
            dataset.chipea_make_flags(args)
        elif args.function == 'md5':
            dataset.chipea_make_md5(args)
        elif args.function == 'soft_deletion':  # 软删除json
            dataset.soft_deletion_json(args)
        elif args.function == 'rename_file_name':  # 自定义格式把文件重命名
            dataset.rename_file_name(args)
        # elif args.function == 'image_check':  # 检查图像问题，包含格式、完整性、通道及其它问题
        #     dataset.image_check(args)
        elif args.function == 'move':  # 自定义格式把文件重命名
            dataset.move_file(args)
        elif args.function == 'modify_flags':  # 修改falgs值
            dataset.modify_flags(args)
        elif args.function == 'filter_flags_true':  # 筛选falgs值为真的标注
            dataset.select_flags_true(args)
        elif args.function == 'ratio':  # 按比例划分测试集和训练集
            dataset.ratio_split(args)


if __name__ == '__main__':
    main()
    # 使用了asyncio.run()函数来启动，协程事件循环，则不需要手动关闭事件循环
    # asyncio.run(main())



# 计算机登录用户: jk
# 系统日期: 2023/7/31 18:05
# 项目名称: chipeak_cv_data_tool
# 开发者: zhanyong

from .base_voc import BaseVoc
from .base_txt import BaseTxt
from .base_sys import BaseSys
from .coco_to_labelme import CocoToLabelme

__all__ = ['BaseVoc', 'BaseTxt', 'BaseSys', 'CocoToLabelme']
